<?php
// $Id: classroom.access.inc,v 1.10 2009/12/24 08:56:07 osoh Exp $

/**
 * Determines if the node can be accessed.
 *
 * @param $nid
 *   Item nid. 
 * @param $roles
 *   Classroom roles. Classroom roles which are allowed to access to the node.
 *   This parameter could be a single role or even an array containing some roles.
 *   Each role could be an integer or a string (which represents an integer) where
 *   '0' = student, '1' = teacher and '2' = role.
 *   If it's NULL, no role checking is performed.
 * @param $type
 *   Type to check against the node.
 */
function classroom_check_access($nid, $roles = NULL, $type = NULL) {
  global $user;

  // Normalize the '$roles' parameter.
  if ($roles !== NULL) {
    $roles_to_normalize = (is_array($roles)) ? $roles : array($roles);
    foreach ($roles_to_normalize as $role) {
      $roles_to_check[] = (int) $role;
    }
  } else {
    $roles_to_check = NULL;
  }

  $node = node_load(array('nid' => $nid));
  if ($node === NULL) {
    return FALSE;
  }

  if (($type !== NULL) && (_classroom_is_type($node->type, $type) == false)) {
    return FALSE;
  }
 
  $course = classroom_get_course_for($node);

  if (user_access('administer classroom')) {
    return TRUE;
  }

  if ($roles_to_check === NULL) { // No role check is needed.
    return classroom_is_user_active($user, $course);
  }

  foreach ($roles_to_check as $role) { // Check the user's role.
    if (classroom_is_user_active($user, $course, (int) $role)) {
      return TRUE;
    }
  }
  return FALSE;
}

/**
 * Get students' permissions.
 *
 * @return
 *   Returns an array containing the added permissions if they were added.
 *   It returns false in case of error.
 */
function classroom_perms() {
  // Student's perms
  $student_perms = array('access classroom');
  $response_types = _classroom_get_types_for('response');
  foreach ($response_types as $type) {
    array_push($student_perms, "create $type->type content");
    array_push($student_perms, "edit own $type->type content");
    array_push($student_perms, "delete own $type->type content");
  }

  // Teachers' perms
  $teacher_perms = array('access classroom');
  $assignment_types = _classroom_get_types_for('assignment');
  foreach ($assignment_types as $type) {
    array_push($teacher_perms, "create $type->type content");
    array_push($teacher_perms, "edit own $type->type content");
    array_push($teacher_perms, "delete own $type->type content");
    array_push($teacher_perms, "edit any $type->type content");
    array_push($teacher_perms, "delete any $type->type content");
  }

  $response_types = _classroom_get_types_for('response');
  foreach ($response_types as $type) {
    array_push($teacher_perms, "edit any $type->type content");
  }

  return array('student' => $student_perms, 'teacher' => $teacher_perms);
}

/**
 * Add permissions to a role.
 *
 * @param $rid
 *   Role ID.
 * @param $perms
 *   Permissions to add.
 * @return
 *   Returns an array containing the added permissions if they were added.
 *   It returns false in case of error.
 */
function classroom_add_perms($rid, $perms = array()) {
  $result = db_query("SELECT * FROM {permission} WHERE rid = %d", $rid);
  $old_perms_obj = db_fetch_object($result);

  if (!$result) {
    return FALSE;
  }

  // Sanitize $perms array.
  // TODO: it must check that permissions are valid.
  $perms = array_unique($perms);

  if ($old_perms_obj) {
    $old_perms = explode(', ', $old_perms_obj->perm);
    $added_perms = array_diff($perms, $old_perms);
    $new_perms = array_merge($old_perms, $added_perms);
    $new_perms_string = implode(', ', $new_perms);
    $result = db_query("UPDATE {permission} SET rid = %d, perm = '%s' WHERE pid = %d", $rid, $new_perms_string, (int) $old_perms_obj->pid);
  } else {
    $result = db_query("INSERT INTO {permission} (rid, perm) VALUES (%d, '%s')",
      $rid, implode(', ', $perms));
    $added_perms = $perms;
  }
  return $added_perms;
}

/**
 * Remove permissions to a role.
 *
 * @param $rid
 *   Role ID.
 * @param $perms
 *   Permissions to remove.
 * @return
 *   Returns an array containing the removed permissions if they were removed.
 *   It returns false in case of error.
 */
function classroom_remove_perms($rid, $perms = array()) {
  $result = db_query("SELECT * FROM {permission} WHERE rid = %d", $rid);
  $old_perms_obj = db_fetch_object($result);

  if (!$result) {
    return FALSE;
  }

  // Sanitize $perms array.
  // TODO: it must check that permissions are valid.
  $perms = array_unique($perms);

  if ($old_perms_obj) {
    $old_perms = explode(', ', $old_perms_obj->perm);
    $removed_perms = array_intersect($perms, $old_perms);
    $new_perms = array_diff($old_perms, $removed_perms);
    $new_perms_string = implode(', ', $new_perms);
    $result = db_query("UPDATE {permission} SET rid = %d, perm = '%s' WHERE pid = %d", $rid, $new_perms_string, (int) $old_perms_obj->pid);
  } else {
    $removed_perms = array();
  }
  return $removed_perms;
}
